let TABLE_NAME = "hljs-ln", LINE_NAME = "hljs-ln-line", CODE_BLOCK_NAME = "hljs-ln-code", NUMBERS_BLOCK_NAME = "hljs-ln-numbers", NUMBER_LINE_NAME = "hljs-ln-n", DATA_ATTR_NAME = "data-line-number", BREAK_LINE_REGEXP = /\r\n|\r|\n/g;
export function lineNumbersInternal(element, options) {
    let internalOptions = mapOptions(element, options);
    duplicateMultilineNodes(element);
    return addLineNumbersBlockFor(element.innerHTML, internalOptions);
}
function mapOptions(element, options) {
    options = options || {};
    return {
        singleLine: getSingleLineOption(options),
        startFrom: getStartFromOption(element, options),
    };
}
function getSingleLineOption(options) {
    let defaultValue = false;
    if (options.singleLine) {
        return options.singleLine;
    }
    return defaultValue;
}
function getStartFromOption(element, options) {
    let defaultValue = 1;
    let startFrom = defaultValue;
    if (isFinite(options.startFrom)) {
        startFrom = options.startFrom;
    }
    let value = getAttribute(element, "data-ln-start-from");
    if (value !== null) {
        startFrom = toNumber(value, defaultValue);
    }
    return startFrom;
}
function getAttribute(element, attrName) {
    return element.hasAttribute(attrName) ? element.getAttribute(attrName) : null;
}
function toNumber(str, fallback) {
    if (!str)
        return fallback;
    let number = Number(str);
    return isFinite(number) ? number : fallback;
}
function duplicateMultilineNodes(element) {
    let nodes = element.childNodes;
    for (let node in nodes) {
        if (nodes.hasOwnProperty(node)) {
            let child = nodes[node];
            if (getLinesCount(child.textContent) > 0) {
                if (child.childNodes.length > 0) {
                    duplicateMultilineNodes(child);
                }
                else {
                    duplicateMultilineNode(child.parentNode);
                }
            }
        }
    }
}
function addLineNumbersBlockFor(inputHtml, options) {
    let lines = getLines(inputHtml);
    if (lines[lines.length - 1] && lines[lines.length - 1].trim() === "") {
        lines.pop();
    }
    if (lines.length > 1 || options.singleLine) {
        let html = "";
        for (let i = 0, l = lines.length; i < l; i++) {
            html += format("<tr>" +
                '<td class="{0} {1}" {3}="{5}">' +
                '<div class="{2}" {3}="{5}"></div>' +
                "</td>" +
                '<td class="{0} {4}" {3}="{5}">' +
                "{6}" +
                "</td>" +
                "</tr>", [
                LINE_NAME,
                NUMBERS_BLOCK_NAME,
                NUMBER_LINE_NAME,
                DATA_ATTR_NAME,
                CODE_BLOCK_NAME,
                i + options.startFrom,
                lines[i].length > 0 ? lines[i] : " ",
            ]);
        }
        return format('<table class="{0}">{1}</table>', [TABLE_NAME, html]);
    }
    return inputHtml;
}
function getLinesCount(text) {
    return ((text && text.trim().match(BREAK_LINE_REGEXP)) || []).length;
}
function duplicateMultilineNode(element) {
    let className = element.className;
    if (!/hljs-/.test(className))
        return;
    let lines = getLines(element.innerHTML);
    for (var i = 0, result = ""; i < lines.length; i++) {
        let lineText = lines[i].length > 0 ? lines[i] : " ";
        result += format('<span class="{0}">{1}</span>\n', [className, lineText]);
    }
    element.innerHTML = result.trim();
}
function getLines(text) {
    if (text.length === 0)
        return [];
    return text.split(BREAK_LINE_REGEXP);
}
function format(format, args) {
    return format.replace(/\{(\d+)\}/g, function (m, n) {
        return args[n] !== undefined ? args[n] : m;
    });
}
export function documentReady(options) {
    try {
        let blocks = document.querySelectorAll("code.hljs,code.nohighlight");
        for (let i in blocks) {
            if (blocks.hasOwnProperty(i)) {
                if (!isPluginDisabledForBlock(blocks[i])) {
                    lineNumbersBlock(blocks[i], options);
                }
            }
        }
    }
    catch (e) {
        window.console.error("LineNumbers error: ", e);
    }
}
function isPluginDisabledForBlock(element) {
    return element.classList.contains("nohljsln");
}
function lineNumbersBlock(element, options) {
    if (typeof element !== "object")
        return;
    element.innerHTML = lineNumbersInternal(element, options);
}
//# sourceMappingURL=utils.js.map